;All rights reserved
;(C) copyright 2002 Gogul Balakrishnan
;---------------------------------------
;This is a part of the OS, Gogul is writing currently. 
;Permission to use, copy, modify and distribute this ;software and documentation is granted without fee, provided ;this copyright message appears on all distributions.I make ;no representations of the suitability of the software for ;any purpose whatsoever. It is provided "as is" without any ;express or implied warranty
;


%include "defs.asm"

%macro  iodelay 0
    jmp %%label1
%%label1:
%endmacro

gdtstseg        equ 01000h
codestseg       equ 02000h
datastseg       equ 05000h

org 100h
bits 16
start:
        mov ax,cs
        mov ds,ax

	mov ah,9h	; Display loading message
	mov dx,loading_msg
	int 21h

	mov ah,3dh	; Get handle for the kernel file
	mov al,00
	mov dx,kernel_file
	int 21h
	jc error

	push ds			;Save ds

	mov bx,ax		;File handle
	mov ax,codestseg
	mov ds,ax		;Load segment address of kernel code into ds
	mov ah,3fh
	mov dx,0		;Load kernel at ds:dx
	mov cx,0ffffh		;Set no of bytes to 64k Change to 
				;exact size later
	int 21h	
	jc error		

	pop ds			; Restore ds 

;Kernel loaded into memory
;Has possibly overwritten dos. So forget dos support

;Move the gdt to the required location
        cld
        mov cx,[gdtsizebytes]
        mov ax,gdtstseg
        mov es,ax
        lea si,[dummy_gdt]
        lea di,[0]
        rep movsb

;Move data to the required location (No Data now)
;        cld
;        mov cx,[datasize]
;        mov ax,datastseg
;        mov es,ax
;        mov word [es:0],cx
;        lea si,[data32]
;        lea di,[2]		; First 2 bytes have length of the string
;        rep movsb

	cli			
;----- SetNewIrqs procedure ----------------------------;
; Reprogram the PIC1 to relocate irq 0-7 to int 20-27h  ;
;	and the PIC2 to relocate irq 8-f to int 28-2fh  ;
;-------------------------------------------------------;
;This is Sathish's code. Need to understand how this is done
SetNewIrqs:

        in al,21h
        mov bl,al
        in al,0a1h
        mov bh,al

        mov al,11h                      ; ICW1, cascade & require ICW4
        out 20h,al
        out 0a0h,al
        iodelay
        mov al,20h                      ; ICW2, switch irqs 0-7 to int 20h
        out 21h,al
        mov al,28h                      ; ICW2, switch irqs 8-F to int 28h
        out 0a1h,al
        iodelay
        mov al,4                        ; ICW3, slave controller on irq2
        out 21h,al
        mov al,2
        out 0a1h,al
        iodelay
        mov al,1                        ; ICW4, normal EOI,nonbuffered, 80x86
        out 21h,al
        out 0a1h,al
        iodelay
        mov al,bl                       ; OCW1, restore interrupt mask
        out 21h,al
        mov al,bh
        out 0a1h,al
	
	;mov eax,[gdtsizebytes]
	;mov [gdtsize],eax

	lgdt [gdtreg]	;Load gdtr
       	lidt [idtreg]	;Load IDTR

;Switch to protected mode and make a far jump to kernel code
        mov eax,cr0
        or al,1
        mov cr0,eax
        jmp dword KERNEL_CS_SEL:0h


error:
	pop ds
	mov ah,9h
	mov dx,panic_msg
	int 21h
	
	mov ah,04ch	;error so exit 
	mov al,0
	int 21h

;Kernel data segment. Nothing important here
data32:
        db 'Kernel loaded. ASL',0
datasize        dw $-data32

;Descriptor Tables and corresponding info
 
gdtreg:
gdtsize dw  0ffffh
gdtbase dd  010000h    

idtreg:
ivtsiz dw  007ffh
ivtbase dd  00500h
	
gdt:
dummy_gdt       dd 0
                dd 0

flatdesc        dd 0000ffffh
                dd 00c09200h

idt_desc        dd 050007ffh  ; Interrupt Vector table 7ff bytes
                dd 00409200h  ; Descriptor table rather

scr_desc        dd 8000ffffh  ; Textmode screen memory
                dd 00c0920bh

code_desc       dd 00000050h  ; Kernel code, data, stack 5x64K 
                dd 00c09a02h

data_desc       dd 00000050h  ; Kernel code,data,stack  5x64K  (DATA alias)
                dd 00c09202h

stk_desc        dd 00000030h  ; Kernel stack 1x64K
                dd 00c09602h     

gdtsizebytes    dw $-gdt

;
;Error messages
;

panic_msg	db  'Error while loading kernel!!!!!!$'
loading_msg	db  'Loading kernel from disk $'
kernel_file	db  'KERNEL.COM',0


;new memory map
;ivt	00500 - 00cffh
;gdt	10000 - 1ffffh
;code	20000 - 6ffffh ,data,stack
;data	50000 - 5ffffh
;stack	60000 - 6ffffh 